@@ -11,7 +11,7 @@ use Carp;
 use JSON::XS;
 use URI;    # The GET requests need URI-escaping
 
-our $VERSION = '0.02_04';
+our $VERSION = '0.03_01';
 
 sub new {
 	my ($class, %opts) = @_;
@@ -69,19 +69,32 @@ sub _send_receive {
 			croak "Unrecognised method '$meth'";
 		}
 
-		if ($self->{ua}->responseCode !~ /[45]00/) {
+		if ($self->{ua}->responseCode !~ /400|50./) {
 			return decode_json $self->{ua}->responseContent
 			  if $self->{ua}->responseHeader ('Content-type') eq
 			  'application/json';
 			return undef;
 		}
 		$tries_left--;
-		carp "REST error " . $self->{ua}->responseCode;
+		warn "REST error " . $self->{ua}->responseCode;
 	}
 
-	carp "Problem with $meth $path";
-	carp "Data was " . encode_json ($data);
-	carp "Client warning: ", $self->{ua}->responseHeader ('Client-Warning');
+	if ($self->{ua}->responseCode > 499) {
+		# Not our problem, so don't carp
+		return {
+			errors => [{
+				status => $self->{ua}->responseCode,
+				detail => $self->{ua}->responseContent //
+					'Server Problem'
+			}]
+		};
+	}
+
+	carp "Problem with $meth $path\nData was " . encode_json ($data) .
+		"\nClient warning: " .
+		(	$self->{ua}->responseHeader ('Client-Warning') //
+			$self->{ua}->responseContent //
+			$self->{ua}->responseCode);
 	return undef;
 }
 
@@ -198,7 +211,7 @@ WebService::AbuseIPDB - Client for the API (version 2) of AbuseIPDB
 
 =head1 DESCRIPTION
 
-L<https://www.abuseipdb.com/|AbuseIPDB> maintains a database of reports
+L<AbuseIPDB|https://www.abuseipdb.com/> maintains a database of reports
 of bad actors on the net. Users may interface with the database through
 a web browser using forms on their site. An alternative is to use their
 API. Version 1 of this API is to be retired in 2020. This module serves
@@ -229,6 +242,15 @@ using this module.
 The timeout in seconds to wait for the server to respond. Defaults to
 20.
 
+Note that the way that L<IO::Socket::IP> handles this in modern
+versions (0.32 and above) means that the timeout applies for each IP
+address which the host resolves to. At time of writing (July 2020) for
+L<api.abuseipdb.com|https://www.sitechecks.co.uk/ipcheck.cgi?in=api.abuseipdb.com>
+there are three IPv4 addresses and three IPv6 addresses so the 20 second
+timeout will apply to each of them consecutively in each supported IP
+family meaning that if all are down you might be waiting for as much as
+2 minutes for a response.
+
 =item retry
 
 The number of times to retry on timeout or network error. Defaults to 0
@@ -353,6 +375,9 @@ L<SendMail::AbuseIPDB> is a client for v1 of the API.
 
 Full documentation for the API is at L<https://docs.abuseipdb.com/>.
 
+Discussion of the IO::Socket::IP timeout can be found in
+L<RT 92075|https://rt.cpan.org/Public/Bug/Display.html?id=92075>.
+
 =head1 AUTHOR
 
 Pete Houston, C<< <cpan at openstrike.co.uk> >>
